/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef CSOCKET_H
#define CSOCKET_H

#include <stdio.h>
#include <iostream>

#include <string.h>
#include <vector>

#if defined(__WIN32__)		/* WIN32 Implementation */
#include <windows.h>
#include <ws2tcpip.h>
#include <winsock2.h>
#else				/* Unix-like Implementation */
#include <stdlib.h>											
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#endif

#include "IStreamDaq.h"

/**
 *\file CSocket.h
 *\date  Created on: 22-apr-2009
 *\author  Author: Claudio Salvadori
 *\brief  Implementation of IStream interface to read a file. 
*	This class reads a file and creates packet shaped 
*         like the SDef structure directives
 */

 /**
 *\class CSocket
  *\brief  Implementation of IStream interface to read a stream from a socket. 
*	This class bytes form socket and creates packet shaped 
*         like the SDef structure directives
 */
class CSocket: implements IStreamDaq{
	public:
		/**\brief CPipe destructor*/		
		~CSocket();

		/**\brief Get the stream definition structure from xml parser and open file
		*\param def 	Filled stream definition structure (input) 
		*\param sty		Style of reading (by length, or by start and end character)
		*\return 	s_ret value about opening the port and about the 
		*/
		virtual s_ret getDefinition (struct SDef def, s_read_style sty);
		
		/**\brief Blocking method to wait the data to be passed to formatter
		 *	  (maybe after a decode and a "divide in message" processes 
		*	  respectively to interpret the escape charactera and to divide
		*	  packets to the correct length).
		*\param buffer 	Pointer to a pointer of a vector of byte to be formatted (output)
		*\param length 	Pointer to the packet lenght variable (output)
		*\return This method return one of the possible value from s_ret \see s_ret*/
		virtual s_ret retStream(unsigned char **buffer, unsigned long *length);

	private:
#if defined(__WIN32__)	/* WIN32 Implementation */
		WSADATA 						so_wsaData;	/**<Structure to initialize winsock interface*/
		SOCKET							so_handle;	/**<Socket handle*/
#else			/* Unix-like Implementation */
		int							so_handle;	/**<Socket handle*/	
#endif
	
		unsigned int						so_Port;	/**<Port number*/
		char*							so_Ip_Addr;	/**<Server IP address*/		
		
		unsigned long						so_Size;	/**< Packet total size */
		unsigned long						so_Size_r;	/**< Received packet total size */
		s_read_style						so_Style;	/**< Reading style*/
		handshake_style						so_HStyle;	/**< Handsaking style*/
		unsigned char						so_Start;	/**< Start character */
		unsigned char						so_End;		/**< End character */
		
		std::vector<struct SHandshaking>*			so_Handshk;	/**< Handshaking packets*/	
		
		/**\brief Connection to a certain port socket like a client
		*\return 	s_ret value about connecting to certain port socket \see s_ret
		*/
		s_ret Socket_init(void);	

		/**\brief Closing the socket connection*/
		void Socket_close(void);							

		/**\brief Receive from socket
		*\param data 		Buffer to receive data
		*\param data_size 	Size of data to receive
		*\return 	s_ret value about receiving from \see s_ret
		*/
		s_ret Socket_read(unsigned char* data, unsigned int data_size);	
		
		/**\brief Send by socket
		*\param data 		Data buffer to send
		*\param data_size 	Size of data to send
		*\return 	s_ret value about sending from \see s_ret
		*/
		s_ret Socket_write(unsigned char* data, unsigned int data_size);	
		
		/**\brief This function execute the handshaking to initialize the stream
		*\param  style 	Handshacking style
		*\return 	s_ret value about sending from \see s_ret
		*/
		s_ret Socket_handshaking(handshake_style style);
};
#endif
