#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */
#include <sys/stat.h>
#include <sys/types.h>

static int serial_des = 0;

int serial_read(unsigned char *data, unsigned int data_size)
{
	int n = read(serial_des, data, sizeof(char)*data_size);
	return n;
}

int serial_write(unsigned char* data, unsigned int data_size)
{
	int n = write(serial_des, data, sizeof(char)*data_size);
	return n;
}

void print_help(void) 
{		
	printf("\nUSAGE: <port_name> <baud_rate>");
	printf("\n       baudrate: 9600, 115200");
	printf("\n\n"); 
}

int main(int argc, char *argv[])
{
	struct termios options;
	speed_t baud;
	char read_char[19005], write_char;
	int cnt;

	if (argc < 3) {
		print_help();
		return -1;
	}
	printf("\nLOG: opening port: %s", argv[1]);

	serial_des = open(argv[1], 
		O_RDWR 		// Read-Write I/O
		| O_NOCTTY 	// Refers to device terminal
		| O_SYNC	// Blocking I/O 
	); 

	if (serial_des == -1) {
		perror("\nERROR: opening port \n");
		return -1;
	} else {
		fcntl(serial_des, F_SETFL, 0);	
	}

	// Get the current options for the port...
	tcgetattr(serial_des, &options);
	// Set the baud rates to...
	switch (atoi(argv[2])) {
	case 9600:
		baud=B9600;
		break;
	case 115200:
		baud=B115200;
		break;
	default:
		print_help();
		return -1;
	}

	cfsetispeed(&options, baud);
	cfsetospeed(&options, baud);

	options.c_cflag = 0x1cb2;
	options.c_oflag = 0x0;
        options.c_iflag = 0x1;
        options.c_lflag = 0x0;
        options.c_line = 0x0;

	// Enable the receiver and set local mode...
// 	options.c_cflag = (CLOCAL | CREAD);
// 	options.c_cflag &= ~PARENB;
// 	options.c_cflag &= ~CSTOPB;
// 	options.c_cflag &= ~CSIZE;
// 	options.c_cflag |= CS8;
// 	options.c_iflag |= IGNBRK | IGNCR;
// 	options.c_cc[VINTR]    = 0;     /* Ctrl-c */                                  
// 	options.c_cc[VQUIT]    = 0;     /* Ctrl-\ */                                  
// 	options.c_cc[VERASE]   = 0;     /* del */                                     
// 	options.c_cc[VKILL]    = 0;     /* @ */                                       
// 	options.c_cc[VEOF]     = 0;     /* Ctrl-d */                                  
// 	options.c_cc[VTIME]    = 0;     /* inter-character timer unused */            
// 	options.c_cc[VMIN]     = 1;     /* blocking read until 1 character arrives */ 
// 	options.c_cc[VSWTC]    = 0;     /* '\0' */                                    
// 	options.c_cc[VSTART]   = 0;     /* Ctrl-q */                                  
// 	options.c_cc[VSTOP]    = 0;     /* Ctrl-s */                                  
// 	options.c_cc[VSUSP]    = 0;     /* Ctrl-z */                                  
// 	options.c_cc[VEOL]     = 0;     /* '\0' */                                    
// 	options.c_cc[VREPRINT] = 0;     /* Ctrl-r */                                  
// 	options.c_cc[VDISCARD] = 0;     /* Ctrl-u */                                  
// 	options.c_cc[VWERASE]  = 0;     /* Ctrl-w */                                  
// 	options.c_cc[VLNEXT]   = 0;     /* Ctrl-v */                                  
// 	options.c_cc[VEOL2]    = 0;     /* '\0' */                             
	// Set the new options for the port...
	tcsetattr(serial_des, TCSANOW, &options);

	printf("\nLOG: start reading\n");
	write_char = '1';
	int count = 0;
	while (1) {
		serial_write((unsigned char *) &write_char, 1);
		fprintf(stderr, "\nSEND [%c]", write_char);
// 		fflush(stdout);
// 		cnt = serial_read((unsigned char *) read_char, 14);
// 		cnt += serial_read((unsigned char *) read_char + 14, 7);
// 		read_char[cnt + 1] = '\0';
// 		printf("\nRECEIVED [%d]: \"%s\"", cnt, read_char);
// 		fflush(stdout);
		while (1) {
		  int n = serial_read((unsigned char *) read_char, 19003);
		  count+= n;
		  printf("%c", read_char[0]);
// 		  if ( count % 100 == 0 )
			  printf("\nCOUNT = %d, %d\n", count, n);
	  
		}
	}
	
	printf("\nLOG: closing port:");
	printf("\n\n");
	close(serial_des);

	return 0;
}

