/*
Copyright (C) 2009 -  Claudio Salvadori
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 *\file CReader.cpp
 *\brief  Reader functions development
 */
#include "CReader.h"

CReader::CReader(){
	re_Source = NULL;
	re_Decoder = NULL;
	re_Message = NULL;
}

CReader::~CReader(){
	delete re_Source;
	
	if(re_Decoder != NULL)
		delete re_Decoder;
		
	if(re_Message != NULL)
		delete re_Message;
}

//This function initialize the class that reads from device
s_ret CReader::getDefinition (struct SDef def){
	s_ret dummy;
	
	switch(def.s_Type){
		case T_SERIAL:
			re_Source = new CSerial();
			break;
		case T_USB:	
			/**\todo usb*/
			break;
		case T_FILE:
			re_Source = new CFile();
			break;
		case T_STDIN:	
			/**\todo stdin*/
			break;
		case T_PIPE:	
			re_Source = new CPipe();
			break;
		case T_SOCKET:	
			re_Source = new CSocket();
			break;
		/**********************************************************/
		/*   ADD HERE THE INITILIZATION OF THE NEW DEVICE CLASS   */
		/**********************************************************/

	}
	
	if(def.s_Option == NULL){
		re_Style = S_STYLE_LENGTH;
	}else{
		re_Style = S_STYLE_S_E;
		
		//Choosing the decoder
		switch(def.s_Option->o_Dec){
			case D_NONE:
			//instantiation of Cmessage (only!!)
			re_Message = new CMessager();
			break;
			
			case D_XOR:
			//instantiation of Xor decoder
			re_Decoder = new CXor();
			//instantiation of Cmessage
			re_Message = new CMessager();
			break;

			case D_AVR:
			//instantiation of AVR decoder
			re_Decoder = new CAVR();
			//instantiation of Cmessage
			re_Message = new CMessager();
			break;
			/***************************************************************/
			/*          ADD HERE THE CASE FOR THE NEW DECODER              */
			/***************************************************************/
		}
	}	
	
	dummy = re_Source->getDefinition(def, re_Style);
	if(dummy != S_RET_OK)
		return dummy;
		
	if(re_Decoder != NULL){
		dummy = re_Decoder->getDefinition(def);
		if(dummy != S_RET_OK)
			return dummy;
	}
	
	if(re_Message != NULL){
		dummy = re_Message->getDefinition(def);
		if(dummy != S_RET_OK)
			return dummy;
	}
		
	return S_RET_OK;
}


//Return a vector of messages
s_ret CReader::retStream(std::vector<struct SMessage> *buffer){
	//il buffer di ritorno se non ci sono i caratteri di controllo  costituito dall'unico elemento letto per lunghezza
	s_ret dummy;
	unsigned char *pkt = NULL;
	unsigned char *msg = NULL;
	unsigned long l_p, l_m = 0;
	
	buffer->clear();
	
	//Receive the packet
	dummy = re_Source->retStream((unsigned char**)&pkt, (unsigned long*)&l_p);
	if(dummy != S_RET_OK)
		return dummy;
	
	#ifdef DEBUG
	cerr << "Packet length:" << l_p << endl;
	for(unsigned long i = 0; i < l_p; i++)
		cerr << "Packet element n " << i << ": " << hex << (int)*(pkt + i) << "(" << *(pkt + i) << ")" << endl;
	#endif
	
	//Decode the packet if it needs	
	if(re_Decoder != NULL){
		dummy = re_Decoder->retStream(pkt, l_p, (unsigned char**)&msg, (unsigned long*)&l_m);
		if(dummy != S_RET_OK){
			delete pkt;
			delete msg;
			return dummy;
		}
		
		delete pkt;
		
		if(re_Message != NULL){
			dummy = re_Message->retStream(msg, l_m, buffer);
			if(dummy != S_RET_OK){
				delete msg;
				return dummy;
			}
		}else{
			struct SMessage help;
			
			//msg coincide with pkt without esc char
			help.me_Size = l_m; 
			help.me_Data = new unsigned char[l_m]; 
			
			for(unsigned long i = 0; i < l_m; i++){
				help.me_Data[i] = pkt[i];
				#ifdef DEBUG
				cerr << "Packet element n " << i << ": " << hex << (int)*(pkt + i) << "(" << *(pkt + i) << ")" << endl;
				#endif
			}
			//memcpy (help.me_Data, msg, l_m);
			
			buffer->push_back(help);
			
			delete msg;
		}
	}else{	
		if(re_Message != NULL){
			dummy = re_Message->retStream(pkt, l_p, buffer);
			if(dummy != S_RET_OK)
				return dummy;
		}else{
			struct SMessage help;
			
			//msg and pkt coincide 
			help.me_Size = l_p; 
			help.me_Data = new unsigned char[l_p]; 
			
			for(unsigned long i = 0; i < l_p; i++){
				help.me_Data[i] = *(pkt + i);
				#ifdef DEBUG
				cerr << "Packet element n " << i << ": " << hex << (int)*(pkt + i) << "(" << *(pkt + i) << ")" << endl;
				#endif
			}
			//memcpy (help.me_Data, pkt, l_p);
			
			buffer->push_back(help);
			
			delete pkt;
		}
	}
	
	#ifdef DEBUG
	for(unsigned long i = 0; i < buffer->size(); i++){
		cerr << "Message length: " << (*buffer)[i].me_Size << endl;
		for(unsigned long j = 0; j < (*buffer)[i].me_Size; j++)
			cerr << "Message n " << i << "element n " << j << ":" << *((*buffer)[i].me_Data + j) << endl;
	}
	#endif
	return S_RET_OK;
}
